/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 * 
 * Copyright (©) 2005,2006,2007,2008 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 2002,2007 Mark Wedel & Crossfire Development Team
 * Copyright (©) 1992,2007 Frank Tore Johansen
 * 
 * Deliantra is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

#ifndef LIBPROTO_H_
#define LIBPROTO_H_

/* anim.c */
extern void free_all_anim();
extern void init_anim();
extern int find_animation(const char *name);
extern void animate_object(object *op, int dir);
/* arch.c */
extern archetype *find_archetype_by_object_name(const char *name);
extern archetype *find_archetype_by_object_type_name(int type, const char *name);
extern object *get_archetype_by_skill_name(const char *skill, int type);
extern archetype *get_archetype_by_type_subtype(int type, int subtype);
extern object *get_archetype_by_object_name(const char *name);
extern object *find_best_weapon_used_match(object *pl, const char *params);
extern int item_matched_string(object *pl, object *op, const char *name);
extern void arch_info(object *op);
extern void clear_archetable();
extern void dump_arch(archetype *at);
extern void dump_all_archetypes();
extern void free_all_archs();
extern void check_generators();
extern bool load_archetype_file(const char *filename);
extern object *create_singularity(const char *name);
extern object *get_archetype(const char *name);
extern unsigned long hasharch(const char *str, int tablesize);
extern archetype *type_to_archetype(int type);
extern object *clone_arch(int type);
extern object *object_create_arch(archetype *at);
/* button.c */
extern void push_button(object *op);
extern void activate_connection (maptile *map, long connection, bool state = true);
extern void update_button(object *op);
extern oblinkpt *get_connection_links (maptile *map, long connection);
extern void use_trigger(object *op);
extern void animate_turning(object *op);
extern int check_altar_sacrifice(const object *altar, const object *sacrifice);
extern int operate_altar(object *altar, object **sacrifice);
extern void trigger_move(object *op, int state);
extern int check_trigger(object *op, object *cause);
extern void add_button_link(object *button, maptile *map, int connected);
extern void remove_button_link(object *op);
extern oblinkpt *get_button_links(const object *button);
extern int get_button_value(const object *button);
extern void do_mood_floor(object *op, object *source = 0);
extern object *check_inv_recursive(object *op, const object *trig);
extern void check_inv(object *op, object *trig);
extern void verify_button_links(const maptile *map);
/* exp.c */
extern int new_exp(const object *ob);
extern int has_ability(const object *ob);
extern void init_experience();
extern void dump_experience();
/* friend.c */
extern void add_friendly_object(object *op);
extern void remove_friendly_object(object *op);
extern void dump_friendly_objects();
extern void clean_friendly_list();
extern int is_friendly(const object *op);
/* glue.c */
extern void fatal(int err);
/* holy.c */
extern void init_gods();
extern void add_god_to_list(archetype *god_arch);
extern int baptize_altar(object *op);
extern godlink *get_rand_god();
extern object *pntr_to_god_obj(godlink *godlnk);
extern void free_all_god();
extern void dump_gods();
/* info.c */
extern void dump_abilities();
extern void print_monsters();
extern void bitstostring(long bits, int num, char *str);
/* image.c */
extern void ReadBmapNames();
extern int FindFace(const char *name, int error);
extern int ReadSmooth();
extern int FindSmooth(uint16 face, uint16 *smoothed);
extern void free_all_images();
/* init.c */
extern void init_library();
extern void init_environ();
extern void init_globals();
extern void init_objects();
extern void init_defaults();
extern void init_dynamic();
extern void init_clocks();
extern void init_attackmess();
/* item.c */
extern int get_power_from_ench(int ench);
extern int calc_item_power(const object *op, int flag);
extern const typedata *get_typedata(int itemtype);
extern const typedata *get_typedata_by_name(const char *name);
extern const char *describe_resistance(const object *op, int newline);
extern const char *query_weight(const object *op);
extern const char *get_levelnumber(int i);
extern const char *get_number(int i);
extern const char *ring_desc(const object *op);
extern const char *query_short_name(const object *op);
extern const char *query_name(const object *op);
extern const char *query_base_name(const object *op, int plural);
extern const char *describe_item(const object *op, object *owner);
extern int is_magical(const object *op);
extern int need_identify(const object *op);
extern void identify(object *op);
/* links.c */
extern objectlink *get_objectlink();
extern oblinkpt *get_objectlinkpt();
extern void free_objectlink(objectlink *ol);
extern void free_objectlinkpt(oblinkpt *obp);
/* living.c */
extern void change_attr_value(living *stats, int attr, sint8 value);
extern void check_stat_bounds(living *stats);
extern int change_abil(object *op, object *tmp);
extern int allowed_class(const object *op);
extern void set_dragon_name(object *pl, const object *abil, const object *skin);
extern void dragon_level_gain(object *who);
extern object *give_skill_by_name(object *op, const char *skill_name);
extern void player_lvl_adj(object *who, object *op);
extern sint64 level_exp(int level, double expmul);
extern void calc_perm_exp(object *op);
extern sint64 check_exp_loss(const object *op, sint64 exp);
extern sint64 check_exp_adjust(const object *op, sint64 exp);
extern void change_exp(object *op, sint64 exp, const char *skill_name, int flag);
extern void apply_death_exp_penalty(object *op);
extern int did_make_save(const object *op, int level, int bonus);
/* los.c */
extern void init_block();
extern void clear_los(player *pl);
extern int has_carried_lights(const object *op);
extern void update_los(object *op);
extern void update_all_map_los(maptile *map);
extern void update_all_los(const maptile *map, int x, int y);
extern void print_los(object *op);
extern void make_sure_seen(const object *op);
extern void make_sure_not_seen(const object *op);
/* map.c */
extern const char *create_pathname (const char *name);
extern int blocked_link(object *ob, maptile *m, int sx, int sy);
extern void fix_container(object *container);
/* object.c */
extern char *dump_object(object *op);
extern void dump_me(object *op, char *outstr);
extern object *get_nearest_part(object *op, const object *pl);
extern object *find_object(tag_t i);
extern object *find_object_name(const char *str);
extern void free_all_object_data();
extern void update_turn_face(object *op);
extern void update_object(object *op, int action);
extern object *merge_ob(object *op, object *top);
extern object *insert_ob_in_map_at(object *op, maptile *m, object *originator, int flag, int x, int y);
extern object *insert_ob_in_map(object *op, maptile *m, object *originator, int flag);
extern void replace_insert_ob_in_map(const char *arch_string, object *op);
extern object *insert_ob_in_ob(object *op, object *where);
extern int check_move_on(object *op, object *originator);
extern object *present_arch(const archetype *at, maptile *m, int x, int y);
extern object *present(unsigned char type, maptile *m, int x, int y);
extern object *present_in_ob(unsigned char type, const object *op);
extern object *present_in_ob_by_name(int type, const char *str, const object *op);
extern object *present_arch_in_ob(const archetype *at, const object *op);
extern void flag_inv(object *op, int flag);
extern void unflag_inv(object *op, int flag);
extern int find_free_spot(const object *ob, maptile *m, int x, int y, int start, int stop);
extern int find_first_free_spot(const object *ob, maptile *m, int x, int y);
extern void get_search_arr(int *search_arr);
extern int find_dir(maptile *m, int x, int y, object *exclude);
extern int distance(const object *ob1, const object *ob2);
extern int find_dir_2(int x, int y);
extern int dirdiff(int dir1, int dir2);
extern int can_see_monsterP(maptile *m, int x, int y, int dir);
extern int can_pick(const object *who, const object *item);
extern object *find_obj_by_type_subtype(const object *who, int type, int subtype);
/* path.c */
extern char *path_combine(const char *src, const char *dst);
extern void path_normalize(char *path);
extern char *path_combine_and_normalize(const char *src, const char *dst);
/* porting.c */
extern void remove_directory(const char *path);
extern char *strcasestr_local(const char *s, const char *find);
extern char *ltostr10(signed long n);
extern char *doubletostr10(double v);
extern FILE *open_and_uncompress(const char *name, int flag, int *compressed);
extern void close_and_delete(FILE *fp, int compressed);
extern void make_path_to_file(char *filename);
/* player.c */
extern void free_player(player *pl);
extern int atnr_is_dragon_enabled(int attacknr);
extern int is_dragon_pl(const object *op);
/* quest.c */
extern int quest_is_quest_marker(const object *marker, int task);
extern int quest_is_in_progress(const object *marker, int task);
extern int quest_is_completed(const object *marker, int task);
extern const char *quest_get_name(const object *marker);
extern object *quest_get_player_quest(const object *pl, const char *name, const char *name_pl);
extern object *quest_get_override(const object *ob, const object *pl);
extern const char *quest_get_override_slaying(const object *ob, const object *pl);
extern const char *quest_get_override_msg(const object *ob, const object *pl);
extern void quest_apply_items(object *wrapper, player *pl);
extern int quest_on_activate(object *ob, player *pl);
extern int quest_is_override_compatible(const object *marker, const object *pl);
/* readable.c */
extern int nstrtok(const char *buf1, const char *buf2);
extern char *strtoktolin(const char *buf1, const char *buf2);
extern int book_overflow(const char *buf1, const char *buf2, int booksize);
extern void init_readable();
extern void change_book(object *book, int msgtype);
extern object *get_random_mon(int level);
extern char *mon_desc(const object *mon);
extern object *get_next_mon(object *tmp);
extern char *mon_info_msg(int level, int booksize);
extern const char *artifact_msg(int level, int booksize);
extern char *spellpath_msg(int level, int booksize);
extern void make_formula_book(object *book, int level);
extern char *msgfile_msg(int level, int booksize);
extern const char *god_info_msg(int level, int booksize);
extern void tailor_readable_ob(object *book, int msg_type);
extern void free_all_readable();
extern void write_book_archive();
extern readable_message_type *get_readable_message_type(object *readable);
/* recipe.c */
extern recipelist *get_formulalist(int i);
extern void init_formulae();
extern void check_formulae();
extern void dump_alchemy();
extern archetype *find_treasure_by_name(const treasure *t, const char *name, int depth);
extern long find_ingred_cost(const char *name);
extern void dump_alchemy_costs();
extern const char *ingred_name(const char *name);
extern int strtoint(const char *buf);
extern artifact *locate_recipe_artifact(const recipe *rp, size_t idx);
extern int numb_ingred(const char *buf);
extern recipelist *get_random_recipelist();
extern recipe *get_random_recipe(recipelist *rpl);
extern void free_all_recipes();
/* region.c */
extern object *get_jail_exit(object *op);
extern void init_regions();
/* shstr.c */
extern int buf_overflow(const char *buf1, const char *buf2, int bufsize);
/* time.c */
extern void reset_sleep();
extern void log_time(long process_utime);
extern int enough_elapsed_time();
extern void sleep_delta();
extern void get_tod(timeofday_t *tod);
extern void print_tod(object *op);
extern void time_info(object *op);
/* treasure.c */
extern bool load_treasure_file(const char *filename);
extern object *generate_treasure(treasurelist *t, int difficulty);
extern int level_for_item(const object *op, int difficulty);
extern int magic_from_difficulty(int difficulty);
extern void set_abs_magic(object *op, int magic);
extern void set_ring_bonus(object *op, int bonus);
extern int get_magic(int diff);
extern void fix_generated_item(object *op, object *creator, int difficulty, int max_magic, int flags);
extern artifactlist *find_artifactlist(int type);
extern void dump_artifacts();
extern void dump_monster_treasure_rec(const char *name, treasure *t, int depth);
extern void dump_monster_treasure(const char *name);
extern void init_artifacts();
extern void add_abilities(object *op, object *change);
extern void give_artifact_abilities(object *op, object *artifct);
extern void generate_artifact(object *op, int difficulty);
extern void fix_flesh_item(object *item, object *donor);
extern int special_potion(object *op);
extern void free_treasurestruct(treasure *t);
extern void free_charlinks(linked_char *lc);
extern void free_artifact(artifact *at);
extern void free_artifactlist(artifactlist *al);
extern void free_all_treasures();
/* utils.c */
extern int random_roll(int min, int max, const object *op, int goodbad);
extern sint64 random_roll64(sint64 min, sint64 max, const object *op, int goodbad);
extern int die_roll(int num, int size, const object *op, int goodbad);
extern materialtype_t *name_to_material (const shstr &name);
extern void transmute_materialname(object *op, const object *change);
extern void set_materialname(object *op, int difficulty, materialtype_t *nmt);
extern void strip_media_tag(char *message);
extern const char *strrstr(const char *haystack, const char *needle);
extern void strip_endline(char *buf);
extern void replace(const char *src, const char *key, const char *replacement, char *result, size_t resultsize);
extern void make_list_like(char *input);
/* loader.c */
extern int set_variable(object *op, char *buf);
extern void init_vars();
extern char *get_ob_diff(object *op, object *op2);
/* map.c */
extern maptile *find_style (const char *dirname, const char *stylename, int difficulty);
extern object *pick_random_object (maptile *style);

/* former funcpoint.h */
/*
 * These function-pointers are defined in common/glue.c
 * The functions used to set and initialise them are also there.
 *
 * Massive change. Those functions are just defined, no callback, and they should be implemented.
 * This means glue.c code & such can go away almost entirely.
 * Ryo 2005-07-15
 */

extern void	move_apply(object *, object *, object *);
extern void	draw_info(int, int, object *, const char *);
extern void	clean_tmp_files(void);
extern void	fix_auto_apply(maptile *);
extern void	init_blocksview_players(void);
extern void	monster_check_apply(object *, object *);
extern void	process_active_maps(void);
extern void	remove_friendly_object(object *);
extern void	update_buttons(maptile *);
extern void	move_teleporter(object *);
extern void	move_firewall(object *);
extern void	move_creator(object *);
extern void move_marker(object *);
extern void	move_duplicator(object *);
extern void trap_adjust(object *, int);
extern void	esrv_send_item(object *, object *);
extern void	esrv_del_item(player *, int);
extern void	esrv_update_item(int, object *, object *);
extern void	esrv_update_spells(player *);
extern void	set_darkness_map(maptile *m);
extern void dragon_ability_gain(object *, int, int);
extern void	weather_effect(const char *);
extern object *	find_skill_by_number(object *, int);

#endif

